{$F+}
UNIT LCntEngn;
{+H
---------------------------------------------------------------------------
  File        - LCNTENGN.PAS

  Purpose     - "Engine" code to measure statistics on Pascal source code.

  Reference   - LCNT program by Bob Phillips (Mitre).

  Revised     - 1992.12.17(KSB)[0.1] Extended to handle more cases by adding
                FSM. Mod'd to handle relative & absolute path references & show
                more stats.
              - 1993.12.20(KSB)[0.2] Added default help message for no command
                line parameters.
                Added -f option which causes file names to be read from a named
                list. Added -d to print out results in report format.
              - 1993.12.21(KSB)[0.3] Fixed some divide by zero bugs.
              - 1998.06.08(KSB)[0.4] Updated code documentation & format.
                Changed counting rules. Blank lines in a comment now count
                as a comment.
              - 1998.07.10(KSB)[0.5] Converted to a unit.
              - 1998.07.14(KSB)[0.6] Added form counting
              - 1998.07.16(KSB)[0.7] Added object counting to forms.
              - 1998.07.17(KSB)[0.8] Fixed code in Get_FormStats.
              - 2000.08.17(KSB)[0.9] Fixed to handle text & binary DFM files.
---------------------------------------------------------------------------}
INTERFACE
USES
  SysUtils, Classes;


TYPE
  TCodeStats  = RECORD
    FiTotal   : LongINT;     {total lines}
    FiBlanks  : LongINT;     {blank lines}
    FiLines   : LongINT;     {source lines}
    FicLines  : LongINT;     {commented source lines}
    FiComments: LongINT;     {comment lines}
  END {RECORD};

  TFormStats  = RECORD
    FiObjects : LongINT;     {no. of objects defined}
    FiSize    : LongINT;     {file size in bytes}
    FiFLines  : LongINT;     {total lines (Form as Text)}
  END {RECORD};

  TLcntEngine = CLASS
  PRIVATE
    FbDetailed: BOOLEAN;
    FbFirst   : BOOLEAN;
    FbFromList: BOOLEAN;
    FiErrorNo : INTEGER;
    FiFileCnt : WORD;
    FiFormCnt : WORD;
    FoReport  : TStringList;
    FoForms   : TStringList;
    FrAllFiles: TCodeStats;
    FrAllForms: TFormStats;
    FrSearch  : TSearchRec;
    FsDirPath : STRING;
    FsFileName: STRING;
    FsThisFile: STRING;
    FIFV      : TEXT;
    PROCEDURE   CountLines(aFileName:STRING;VAR aStats:TCodeStats);
    PROCEDURE   FRM_Summary;
    PROCEDURE   FRM_DetailBar;
    PROCEDURE   FRM_DetailHdr;
    PROCEDURE   FRM_DetailFtr;
    PROCEDURE   FRM_DetailLine(aFileName:STRING; aStats:TFormStats);
    FUNCTION    Get_FormStats(aFileName:STRING; VAR aStats:TFormStats):BOOLEAN;
    FUNCTION    GetName:STRING;
    FUNCTION    NotAnError:BOOLEAN;
    PROCEDURE   RPT_Summary;
    PROCEDURE   RPT_DetailBar;
    PROCEDURE   RPT_DetailHdr;
    PROCEDURE   RPT_DetailFtr;
    PROCEDURE   RPT_DetailLine(aFileName:STRING;aStats:TCodeStats);
    PROCEDURE   SetFileName(aValue:STRING);
  PUBLIC
    CONSTRUCTOR Create;
    DESTRUCTOR  Destroy; OVERRIDE;
    FUNCTION    Execute: BOOLEAN;
  PUBLIC
    PROPERTY    FileName        :STRING      READ FsFileName WRITE SetFileName;
    PROPERTY    IsFileList      :BOOLEAN     READ FbFromList WRITE FbFromList;
    PROPERTY    IsDetailedReport:BOOLEAN     READ FbDetailed WRITE FbDetailed;
    PROPERTY    Report          :TStringList READ FoReport;
  END {CLASS};


{}FUNCTION GetVersion:STRING;
              {=================================================}
IMPLEMENTATION
USES
  KSString;


CONST
  csVersion   = '(Win32 v:0.9)';
  ciMaxFileNameWidth    = 14;
  ciMaxFormNameWidth    = 20;

VAR
  usPath      : STRING;


{}PROCEDURE   AccumulateFileTotals(a1FileCount: TCodeStats;VAR aAllFileCount: TCodeStats);
{+H
---------------------------------------------------------------------------
  Purpose     - Add line counts to a summary (aAllFileCount) record.
---------------------------------------------------------------------------}
  BEGIN
    Inc(aAllFileCount.FiTotal,a1FileCount.FiTotal);
    Inc(aAllFileCount.FiBlanks,a1FileCount.FiBlanks);
    Inc(aAllFileCount.FiLines,a1FileCount.FiLines);
    Inc(aAllFileCount.FicLines,a1FileCount.FicLines);
    Inc(aAllFileCount.FiComments,a1FileCount.FiComments);
{}END {AccumulateFileTotals};




{}PROCEDURE   AccumulateFormTotals(a1FormCount: TFormStats;VAR aAllFormCount: TFormStats);
{+H
---------------------------------------------------------------------------
  Purpose     - Add form counts to a summary (aAllFormCount) record.
---------------------------------------------------------------------------}
  BEGIN
    Inc(aAllFormCount.FiFLines,  a1FormCount.FiFLines);
    Inc(aAllFormCount.FiSize,   a1FormCount.FiSize);
    Inc(aAllFormCount.FiObjects,a1FormCount.FiObjects);
{}END {AccumulateFormTotals};




{}FUNCTION    FormatInt(aValue,aWidth:LongINT):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Provide simple integer string formatting.
---------------------------------------------------------------------------}
  BEGIN
    Str(aValue:aWidth,result);
{}END {FormatInt};




{}FUNCTION    GetVersion:STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Return the version string.
---------------------------------------------------------------------------}
  BEGIN
    result := csVersion;
{}END {GetVersion};




{}FUNCTION    PercentOfSource(aValue:INTEGER;aStats:TCodeStats):INTEGER;
{+H
---------------------------------------------------------------------------
  Purpose     - Compute the percent of source.
---------------------------------------------------------------------------}
  BEGIN
    WITH aStats DO BEGIN
      PercentOfSource := Round(aValue/FiLines*100.0);
    END {WITH};
{}END {PercentOfSource};




{}FUNCTION    PercentOfTotal(aValue:INTEGER;aStats:TCodeStats):INTEGER;
{+H
---------------------------------------------------------------------------
  Purpose     - Compute the percent of total.
---------------------------------------------------------------------------}
  BEGIN
    WITH aStats DO BEGIN
      PercentOfTotal := Round(aValue/FiTotal*100.0);
    END {WITH};
{}END {PercentOfTotal};




{}PROCEDURE   TLcntEngine.CountLines(aFileName:STRING;VAR aStats:TCodeStats);
{+H
---------------------------------------------------------------------------
  Purpose     - In the named file, count the following:
                  1. the total number of lines
                  2. the number of source lines
                  3. the number of comment lines
                  4. the number of commented source lines
                  5. the number of blank lines
                However, "bubbles" (a open brace immediately followed by
                close brace) are not counted.
  Author      - Keith S. Brown
  Revised     - 1998.0711 (wrote first version).
              - 1998.0712 (KSB) fixed bug with parenthesis star comments.
              - 1998.0713 (KSB) Added code for slash slash comments (delphi).
---------------------------------------------------------------------------}
  TYPE
    TState    = (_c1,_c2,_other);  {states for Finite State Machine}
  CONST
    ch1stPrintChar = #33;     {first printable character is "!"}
    chBraceClose   = '}';
    chBraceOpen    = '{';
    chParenClose   = ')';
    chParenOpen    = '(';
    chQuote        = '''';
    chSlash        = '/';
    chStar         = '*';
    cBubble        = chBraceOpen+chBraceClose;
    cParenStar     = chParenOpen+chStar;
    cStarParen     = chStar+chParenClose;
  VAR
    lbCmmntFnd: BOOLEAN;     {signals that a comment (either kind) was found}
    lbSrcFnd  : BOOLEAN;     {signals that a source line was found}
    leState   : TState;      {records the type of comment found (_c1 is brace)}
    li        : WORD;        {general use}
    ls        : STRING;      {general use}
    lTXT      : TEXT;        {file being read from}
  BEGIN       {Initialize the counts}
    FillChar(aStats,SizeOf(aStats),0);
              {Open the file for reading}
    Assign(lTxt,aFileName);
              {turn off I/O checking before resetting file}
    {$I-}
    Reset(lTxt);
    {$I+}
              {IOresult returns non-zero if an I/O error occurred}
    IF IOresult <> 0 THEN BEGIN
      FoReport.Add('File "'+aFileName+'" not found.');
      Exit;
    END {IF};
              {Read and do a minimal syntax parse, counting the
               source lines, comment lines and blank lines}
    leState := _other;
    WITH aStats DO BEGIN
      WHILE NOT EOF(lTxt) DO BEGIN
        ReadLn(lTxt,ls);
        Inc(FiTotal);
        lbCmmntFnd := FALSE;
        lbSrcFnd := FALSE;
              {Delete any leading nonprinting characters from the line}
        WHILE(Length(ls) > 0) AND (ls[1]<ch1stPrintChar) DO BEGIN
          Delete(ls,1,1);
        END {WHILE};

        IF (Length(ls) > 0) THEN BEGIN
              {Delete any nonprinting characters (including spaces)}
          li := 1;
          WHILE(li <= Length(ls)) DO BEGIN
            IF ls[li]<ch1stPrintChar THEN
              Delete(ls,li,1)
            ELSE BEGIN
              Inc(li);
            END {IF};
          END {WHILE};

              {Delete "bubbles" from code before counting comments}
          li := Pos(cBubble,ls);
          WHILE(li > 0) DO BEGIN
            Delete(ls,li,2);
            li := Pos(cBubble,ls);
          END {WHILE};

              {Handle the 3 cases: a "brace", a "(*", anything else}
              {If this line is a continued comment, force it to that
               kind of comment by prepending the appropriate delimiter
               to the line read from the file.}
          CASE leState OF
            _c1    : ls := chBraceOpen+ls;
            _c2    : ls := cParenStar+ls;
          END {CASE};

          WHILE Length(ls) > 0 DO BEGIN
              {examine the first character}
            CASE ls[1] OF
              {if it is a open brace, we have a "c1" comment}
              chBraceOpen    : BEGIN
                lbCmmntFnd := TRUE;
                leState := _c1;

              {look for the end of the comment on this line}
                REPEAT
                  Delete(ls,1,1);
                  IF (Length(ls) > 0) AND (ls[1] = chBraceClose) THEN BEGIN
                    leState := _other;
                    Delete(ls,1,1);
                    Break;
                  END {IF};
                UNTIL Length(ls) = 0;

              {If leState is still _c1 then the end of the comment
               was not found -- therefore it is a continued comment.
               If anything remains in the string, re-examine the
               first character by calling Continue to jump to the
               loop start.

               If the string is empty, then exit the loop by calling
               Break so the next line can be read.}
                IF Length(ls) > 0 THEN BEGIN
                  Continue;
                END ELSE BEGIN
                  Break;
                END {IF};
              END {chBraceOpen};

              chParenOpen    : BEGIN
              {if it is a open parenthesis followed by an asterisk, we
               have a "c2" comment}
                IF (Length(ls) > 1) AND (ls[2] = chStar) THEN BEGIN
                  lbCmmntFnd := TRUE;
                  leState := _c2;

              {look for the end of the comment on this line}
                  Delete(ls,1,2);
                  REPEAT
                    IF Pos(cStarParen,ls) > 0 THEN BEGIN
                      leState := _other;
                      Delete(ls,1,Pos(cStarParen,ls)+2);
                      Break;
                    END ELSE BEGIN
                      Delete(ls,1,1);
                    END {IF};
                  UNTIL Length(ls) = 0;

              {If leState is still _c2 then the end of the comment
               was not found -- therefore it is a continued comment.
               If anything remains in the string, re-examine the
               first character by calling Continue to jump to the
               loop start.

               If the string is empty, then exit the loop by calling
               Break so the next line can be read.}
                  IF Length(ls) > 0 THEN BEGIN
                    Continue;
                  END ELSE BEGIN
                    Break;
                  END {IF};
                END ELSE BEGIN
              {an open parenthesis followed by any other character is
               not a comment, so set the state to "other"}
                  leState := _other;
                END {IF};
              END {chParenOpen};

              chSlash   : BEGIN
              {if it is a slash followed by another slash, we have a "c3" comment.
               a "c3" comment is from the two slashes to the end of the line...
               it is never continued to the next line. Therefore we set the state
               to "other" as the comment state should not persist...}
                leState := _other;
                IF (Length(ls) > 1) AND (ls[2] = chSlash) THEN BEGIN
                  lbCmmntFnd := TRUE;
              {"consume" the rest of the line}
                  ls := '';
              {exit the loop by calling Break so the next line can be read}
                  Break;
                END ELSE BEGIN
              {a slash followed by any other character is not a comment,
               the current character is deleted at the bottom of the loop
               so we do nothing at this point}
                END {IF};
              END {chSlash};

              chQuote   : BEGIN
              {we discard all quoted literals because a comment delimiter
               might be embedded in one. If quoted literals do not
               received special processing, our counts may be incorrect.}
                lbSrcFnd:= TRUE;
                leState := _other;

              {"consume" the quoted literal}
                Delete(ls,1,1);
                WHILE(Length(ls) > 0) DO BEGIN
                  IF ls[1] = chQuote THEN BEGIN
                    IF (Length(ls) > 1) AND (ls[2] = chQuote) THEN BEGIN
                      {do nothing}
                    END ELSE BEGIN
                      Delete(ls,1,1);
                      Break;
                    END {IF};
                  END {IF};

                  Delete(ls,1,1);
                END {WHILE};
              END {chQuote};

              ELSE BEGIN
              {any other character should be from a non-blank source line}
                leState := _other;
                lbSrcFnd := TRUE;
              END {ELSE};
            END {CASE};

              {delete the current character so we can examine the next one}
            Delete(ls,1,1);
          END {WHILE};

              {Tally up the source lines, comment lines and commented
               source lines found thus far}
          IF lbCmmntFnd THEN BEGIN
            IF lbSrcFnd THEN BEGIN
              Inc(FicLines);
            END ELSE BEGIN
              Inc(FiComments);
            END {IF};
          END {IF};

          IF lbSrcFnd THEN BEGIN
            Inc(FiLines);
          END {IF};
        END ELSE BEGIN
              {a blank line was found, is it part of a comment or
               a true "blank" line? If leState indicates that we
               are still processing a "c1" or "c2" comment, then
               we increment the count for comments. Otherwise we
               bump the count for true blank lines.}
          CASE leState OF
            _c1    : Inc(FiComments);
            _c2    : Inc(FiComments);
            _other : Inc(FiBlanks);
          END {CASE};
        END {IF};
      END {WHILE};
    END {WITH};

              {Close the source file before returning}
    Close(lTxt);
{}END {CountLines};




{}CONSTRUCTOR TLcntEngine.Create;
{+M
---------------------------------------------------------------------------
  Purpose     - Initialize the engine.
---------------------------------------------------------------------------}
  BEGIN
    FbDetailed := FALSE;
    FbFromList := FALSE;
    FoReport := TStringList.Create;
    FoForms  := TStringList.Create;
{}END {Create};




{}DESTRUCTOR  TLcntEngine.Destroy;
{+M
---------------------------------------------------------------------------
  Purpose     - Release memory.
---------------------------------------------------------------------------}
  BEGIN
    FoReport.Free;
    FoForms.Free;
{}END {Destroy};




{}FUNCTION    TLcntEngine.Execute:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     - Execute performs the line count function and returns true
                if performed successfully.
---------------------------------------------------------------------------}
  VAR
    lbHasForm : BOOLEAN;
    li        : INTEGER;
    lrOneFile : TCodeStats;
    lrOneForm : TFormStats;
    lsFileName: STRING;
  BEGIN
    result := FALSE;
    FbFirst  := TRUE;
    FiFileCnt:= 0;
    TRY
      IF FbFromList THEN BEGIN
        Assign(FIFV,FsFileName);
        Reset(FIFV);
      END {IF};

      WITH FoReport DO BEGIN
        Clear;
        Add('W32LCnt '+csVersion+'     Copyright (c) 1992-2000 by RP & KSB');
        Add('  Count the number of blank lines, comment lines, source lines');
        Add('  and commented source lines in a Pascal (or Delphi) source file.');
        Add('');
        Add('  Note: "{}" also called "bubbles" are not counted.');
        Add('  [A "bubble" is an open brace immediately followed by a close');
        Add('  brace. They are normally used as spacers for nested routines]');
        Add('');
      END {WITH};

              {Initialize the form related variables}
      FiFormCnt := 0;
      FoForms.Clear;

              {Write the detailed report header}
      IF FbDetailed THEN BEGIN
        RPT_DetailHdr;
        FRM_DetailHdr;
      END {IF};

      FillChar(FrAllFiles,SizeOf(FrAllFiles),0);
      FillChar(FrAllForms,SizeOf(FrAllForms),0);
      lsFileName := GetName;

      WHILE NotAnError DO BEGIN
        CountLines(lsFileName,lrOneFile);
        AccumulateFileTotals(lrOneFile,FrAllFiles);
        Inc(FiFileCnt);
              {Get form statistics}
        lbHasForm := Get_FormStats(lsFileName,lrOneForm);
        IF lbHasForm THEN BEGIN
          AccumulateFormTotals(lrOneForm,FrAllForms);
          Inc(FiFormCnt);
        END {IF};

              {Write one line to the detailed report}
        IF FbDetailed THEN BEGIN
          RPT_DetailLine(lsFileName,lrOneFile);
          IF lbHasForm THEN BEGIN
            FRM_DetailLine(lsFileName,lrOneForm);
          END {IF};
        END {IF};

              {get the next file name to process}
        lsFileName := GetName;
      END {WHILE};

              {complete the report}
      IF FbDetailed THEN BEGIN
        RPT_DetailFtr;
        IF FiFormCnt > 0 THEN BEGIN
          FRM_DetailFtr;
        END {IF};
      END ELSE BEGIN
        RPT_Summary;
        IF FiFormCnt > 0 THEN BEGIN
          FRM_Summary;
        END {IF};
      END {IF};

              {Copy the forms statistics to the report}
      FoReport.Add('');
      IF FiFormCnt > 0 THEN BEGIN
        FOR li := 0 TO FoForms.Count-1 DO BEGIN
          FoReport.Add(FoForms[li]);
        END {IF};

        FoForms.Clear;
      END {IF};
    FINALLY
      IF NOT FbFirst THEN BEGIN
        FindClose(FrSearch);
      END {IF};
    END {TRY};

    result := TRUE;
{}END {Execute};




{}PROCEDURE   TLcntEngine.FRM_DetailBar;
{+M
---------------------------------------------------------------------------
  Purpose     - Writes detailed form report divider bars.
---------------------------------------------------------------------------}
{
                    FORMS Detailed Report
          name      Objects        total         size
--------------     --------     --------     --------
    sample.frm           25          112         2075
}
  VAR
    ls        : STRING;
  BEGIN
    ls := '------------';
    WHILE Length(ls) < ciMaxFormNameWidth DO BEGIN
      ls := '-' + ls;
    END {WHILE};

    FoForms.Add(ls+'     --------     --------     --------');
{}END {FRM_DetailBar};




{}PROCEDURE   TLcntEngine.FRM_DetailHdr;
{+H
---------------------------------------------------------------------------
  Purpose     - Detailed report header for forms.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    WITH FoForms DO BEGIN
      Add('                           FORMS Detailed Report');
      ls := 'name';
      WHILE Length(ls) < ciMaxFormNameWidth DO BEGIN
        ls := ' ' + ls;
      END {WHILE};

      Add(ls+'      Objects        Lines         Size');
      FRM_DetailBar;
    END {WITH};
{}END {FRM_DetailHdr};




{}PROCEDURE   TLcntEngine.FRM_DetailFtr;
{+H
---------------------------------------------------------------------------
  Purpose     - Detailed report footer for forms.
---------------------------------------------------------------------------}
  BEGIN
    FRM_DetailBar;
    FRM_DetailLine('TOTAL',FrAllForms);
    WITH FoForms DO BEGIN
      Add('');
      Add(' '+IntToStr(FiFormCnt)+' forms processed.');
    END {WITH};
{}END {FRM_DetailFtr};




{}PROCEDURE   TLcntEngine.FRM_DetailLine(aFileName:STRING; aStats:TFormStats);
{+H
---------------------------------------------------------------------------
  Purpose     - Detail report for forms.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    IF (Pos(':',aFileName) > 0) OR (Pos('\',aFileName) > 0) THEN BEGIN
      ls := ExtractFileName(aFileName);
    END ELSE BEGIN
      ls := aFileName;
    END {IF};

    WITH aStats,FoForms DO BEGIN
      IF UpperCase(ExtractFileExt(aFileName)) = '.PAS' THEN BEGIN
        ls := ChangeFileExt(ls,'.dfm');
      END {IF};

      IF Length(ls) > ciMaxFormNameWidth THEN BEGIN
        Add(ls);
        ls := '';
      END {IF};

      WHILE Length(ls) < ciMaxFormNameWidth DO BEGIN
        ls := ' '+ls;
      END {WHILE};

      ls := ls +'     '+FormatInt(FiObjects,08);
      ls := ls +'     '+FormatInt(FiFLines,08);
      ls := ls +'     '+FormatInt(FiSize,08);
      Add(ls);
    END {WITH};
{}END {FRM_DetailLine};




{}PROCEDURE   TLcntEngine.FRM_Summary;
{+H
---------------------------------------------------------------------------
  Purpose     - Summary report for forms.
---------------------------------------------------------------------------}
  BEGIN
    WITH FrAllForms,FoForms DO BEGIN
      Add('');
      Add('Form Count Summary Report for: ');
      Add('    '+FsFileName);
      IF FiFileCnt > 1 THEN BEGIN
        Add('    (a list file containing '+IntToStr(FiFormCnt)+' form names)');
      END {IF};

      Add('');
      Add('                category      value');
      Add('------------------------     ------');
      Add('           Total Objects     '+FormatInt(FiObjects,6));
      Add('        Equivalent Lines     '+FormatInt( FiFLines,6));
      Add('        Size (all Forms)     '+FormatInt(   FiSize,6)+'  bytes');
    END {WITH};
{}END {FRM_Summary};




{}FUNCTION    TLcntEngine.Get_FormStats(aFileName:STRING; VAR aStats:TFormStats):BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     - Get the required data from the .DFM file.
                (the file size, the number of objects and number of line
                equivalents.)
  Revised     - 1998.0717 (KSB) Fixed object and line count bug.
              - 2000.08.17(KSB) Handled text .dfm files.
---------------------------------------------------------------------------}
{}{}PROCEDURE Get_DFM_Stats(asFileName:STRING);
    VAR
      lTXT    : TEXTFILE;
      lsBuffer: STRING;
    BEGIN
      Assign(lTXT,asFileName);
      Reset(lTXT);
      TRY
        WHILE NOT Eof(lTXT) DO BEGIN
          ReadLn(lTXT,lsBuffer);
          Inc(aStats.FiFLines);
          lsBuffer := UpperCase(Trim(lsBuffer));
          IF Parse(lsBuffer,' ') = 'OBJECT' THEN BEGIN
            Inc(aStats.FiObjects);
          END {IF};
        END {IF};
      FINALLY
        CloseFile(lTXT);
      END {TRY};
{}{}END {Get_DFM_Stats};


  VAR
    laBuf     : ARRAY[0..2]OF BYTE;
    lbIsBinary: BOOLEAN;
    lF        : FILE;
    liCount   : INTEGER;
    loDFMstrm : TFileStream;
    loTXTstrm : TFileStream;
  BEGIN
    aFileName := ChangeFileExt(aFileName,'.DFM');
    result := FileExists(aFileName);
    IF result THEN BEGIN
      Assign(lF,aFileName);
      TRY
        Reset(lF,1);
        aStats.FiSize := FileSize(lF);
        aStats.FiObjects := 0;
        aStats.FiFLines := 0;
        BlockRead(lF,laBuf,3,liCount);
      FINALLY
        CloseFile(lF);
      END {TRY};

      IF (liCount = 3) THEN BEGIN
        lbIsBinary := (laBuf[0] = $FF) AND(laBuf[1] = $0A) AND(laBuf[2] = $00);
      END ELSE BEGIN
        CloseFile(lF);
        Exit;
      END {IF};

      IF lbIsBinary THEN BEGIN
        loDFMstrm := TFileStream.Create(aFileName,fmOpenRead OR fmShareCompat);
        TRY
              {needs to be "fmCreate" or the file will not be created if it does not exist}
          loTXTstrm := TFileStream.Create('W32LCnt.TXT',fmCreate OR fmShareExclusive);
          TRY
            ObjectResourceToText(loDFMstrm,loTXTstrm);
          FINALLY
            loTXTstrm.Free;
          END {TRY};

          Get_DFM_Stats('W32LCnt.TXT');
        FINALLY
          loDFMstrm.Free;
        END {TRY};
      END ELSE BEGIN
        Get_DFM_Stats(aFileName);
      END {IF};
    END {IF};
{}END {Get_FormStats};




{}FUNCTION    TLcntEngine.GetName:STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Get the name of the next file to process.
---------------------------------------------------------------------------}
  BEGIN
    IF FbFromList THEN BEGIN
      REPEAT
        IF NOT EOF(FIFV) THEN BEGIN
          ReadLn(FIFV,FsThisFile);
          FsThisFile := Trim(FsThisFile);
        END ELSE BEGIN
          FsThisFile := '';
          {$I-}
          Close(FIFV);
          {$I+}
          Exit;
        END {IF};
      UNTIL FsThisFile <> '';
    END ELSE BEGIN
      IF FbFirst THEN BEGIN
        FiErrorNo := FindFirst(FsThisFile,faAnyFile,FrSearch);
        FbFirst := FALSE;
      END ELSE BEGIN
        FiErrorNo := FindNext(FrSearch);
      END {IF};

      FsThisFile := FsDirPath+FrSearch.NAME;
    END {IF};

    result := FsThisFile;
{}END {GetName};




{}FUNCTION    TLcntEngine.NotAnError:BOOLEAN;
{+H
---------------------------------------------------------------------------
  Purpose     - Provides termination condition.
---------------------------------------------------------------------------}
  BEGIN
    IF FbFromList THEN BEGIN
      NotAnError := FsThisFile <> ''
    END ELSE BEGIN
      NotAnError := FiErrorNo = 0;
    END {IF};
{}END {NotAnError};




{}PROCEDURE   TLcntEngine.RPT_DetailBar;
{+H
---------------------------------------------------------------------------
  Purpose     - Writes detailed report divider bars.
---------------------------------------------------------------------------}
{
                           LCNT (v:0.2) Detailed Report
                                            all    commented
name             Blank  %   comment  %    source  %    source      %S    total
-------------- ------- --- -------- --- -------- --- -------- --- --- --------
sample.pas          25 xxx       50 xxx    20000 xxx       15 xxx         2075
}
  VAR
    ls        : STRING;
  BEGIN
    ls := '------------';
    WHILE Length(ls) < ciMaxFileNameWidth DO BEGIN
      ls := '-' + ls;
    END {WHILE};

    FoReport.Add(ls+' -------- --- -------- --- -------- --- -------- --- --- --------');
{}END {RPT_DetailBar};




{}PROCEDURE   TLcntEngine.RPT_DetailFtr;
{+H
---------------------------------------------------------------------------
  Purpose     - Writes detailed report footer.
---------------------------------------------------------------------------}
  BEGIN
    RPT_DetailBar;
    RPT_DetailLine('TOTAL',FrAllFiles);
    WITH FoReport DO BEGIN
      Add('');
      Add(' '+IntToStr(FiFileCnt)+' files processed.');
      Add(' %S :: per cent of source lines containing inline comments.');
    END {WITH};
{}END {RPT_DetailFtr};




{}PROCEDURE   TLcntEngine.RPT_DetailHdr;
{+H
---------------------------------------------------------------------------
  Purpose     - Writes detailed report header.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    WITH FoReport DO BEGIN
      Add('                           Line Count Detailed Report');
      Add('                                             all    commented');
      ls := 'name';
      WHILE Length(ls) < ciMaxFileNameWidth DO BEGIN
        ls := ' ' + ls;
      END {WHILE};

      Add(ls+'    Blank  %   comment  %    source  %    source   %  %S    total');
      RPT_DetailBar;
    END {WITH};
{}END {RPT_DetailHdr};




{}PROCEDURE   TLcntEngine.RPT_DetailLine(aFileName:STRING;aStats:TCodeStats);
{+H
---------------------------------------------------------------------------
  Purpose     - Writes detailed report detail line.
  Revised     - 1998.0708 (KSB) Restructured. Mod'd to handle revised
                counting rules.
              - 2000.08.24(KSB) Added paths to report.
---------------------------------------------------------------------------}
  VAR
    lbChanged : BOOLEAN;
    ls        : STRING;
    lsPath    : STRING;
  BEGIN
    IF (Pos(':',aFileName) > 0) OR (Pos('\',aFileName) > 0) THEN BEGIN
      ls := ExtractFileName(aFileName);
    END ELSE BEGIN
      ls := aFileName;
    END {IF};

    lsPath := ExtractFilePath(aFileName);
    IF lsPath <> usPath THEN BEGIN
      lbChanged := TRUE;
      usPath := lsPath;
    END {IF};

    WITH aStats,FoReport DO BEGIN
      IF lbChanged THEN BEGIN
        Add('PATH: '+lsPath);
      END {IF};

      IF Length(ls) > ciMaxFileNameWidth THEN BEGIN
        Add(ls);
        ls := '          ';
      END {IF};

      WHILE Length(ls) < ciMaxFileNameWidth DO BEGIN
        ls := ' '+ls;
      END {WHILE};

      ls := ls +' ';
      IF FiTotal > 0 THEN BEGIN
        ls := ls + FormatInt(  FiBlanks,08)+' '+FormatInt(PercentOfTotal(FiBlanks,aStats)  ,3)+' ';
        ls := ls + FormatInt(FiComments,08)+' '+FormatInt(PercentOfTotal(FiComments,aStats),3)+' ';
        ls := ls + FormatInt(   FiLines,08)+' '+FormatInt(PercentOfTotal(FiLines,aStats)   ,3)+' ';
        ls := ls + FormatInt(  FicLines,08)+' '+FormatInt(PercentOfTotal(FicLines,aStats)  ,3)+' ';
      END ELSE BEGIN
        ls := ls + FormatInt(0,08)+' --- '+FormatInt(0,08)+' --- '+FormatInt(0,08)+' --- '+FormatInt(0,08)+' --- ';
      END {IF};

      IF (FiLines) > 0 THEN BEGIN
        ls := ls + FormatInt(PercentOfSource(FicLines,aStats),3)+' ';
      END ELSE BEGIN
        ls := ls + '--- ';
      END {IF};

      Add(ls+FormatInt(FiTotal,08));
    END {WITH};
{}END {RPT_DetailLine};




{}PROCEDURE   TLcntEngine.RPT_Summary;
{+H
---------------------------------------------------------------------------
  Purpose     - Print out summarized statistics.
---------------------------------------------------------------------------}
  BEGIN
    WITH FrAllFiles,FoReport DO BEGIN
      Add('');
      Add('Line Count Summary Report for: ');
      Add('    '+FsFileName);
      IF FiFileCnt > 1 THEN BEGIN
        Add('    (a list file containing '+IntToStr(FiFileCnt)+' file names)');
      END {IF};

      Add('    type of line      count    % of total');
      Add('    ------------     ------    ----------');
      IF FiTotal > 0 THEN BEGIN
        Add('           Blank     '+FormatInt(  FiBlanks,6)+'        '+FormatInt(PercentOfTotal(FiBlanks,FrAllFiles),6));
        Add('         Comment     '+FormatInt(FiComments,6)+'        '+FormatInt(PercentOfTotal(FiComments,FrAllFiles),6));
        Add('      All Source     '+FormatInt(   FiLines,6)+'        '+FormatInt(PercentOfTotal(FiLines,FrAllFiles),6));
        IF FiLines > 0 THEN BEGIN
          Add('Commented source         ** '+FormatInt(FicLines,6)+' '+FormatInt(PercentOfTotal(FicLines,FrAllFiles),6)
            + '   (is '+FormatInt(PercentOfSource(FicLines,FrAllFiles),2)+'% of All Source)');
        END ELSE BEGIN
          Add('');
        END {IF};
      END {IF};

      Add('                     ======');
      Add('     Total lines:    '+FormatInt(FiTotal,6));
    END {WITH};
{}END {RPT_Summary};




{}PROCEDURE   TLcntEngine.SetFileName(aValue:STRING);
{+H
---------------------------------------------------------------------------
  Purpose     - Do not allow DFM files to be directly processed.
  Revised     - 1998.0818 (KSB) Added DFM protection.
---------------------------------------------------------------------------}
  BEGIN
    IF UpperCase(ExtractFileExt(aValue)) = '.DFM' THEN BEGIN
      aValue := ChangeFileExt(aValue,'.pas');
    END {IF};

    FsFileName := aValue;
    FsThisFile := FsFileName;
    FsDirPath  := ExtractFilePath(aValue);
{}END {SetFileName};




INITIALIZATION
  usPath := 'A:\';
END {LCntEngn}.
