UNIT UnitTree;
{+F
---------------------------------------------------------------------------
  File        - UNITTREE.PAS
  Purpose     - Given a file name (a DPR or PAS file) prepare a unit tree
                or inverted unit tree listing.
  Author      - Keith S. Brown
  Copyright    1991..2000 by KSS (Keith S. Brown)
  Revised     - 2000.08.20(KSB) extracted unit tree code from LST7 (1991).
---------------------------------------------------------------------------}
INTERFACE
USES
  Classes,
  KDStruct;   {binary tree data structure}

  
TYPE
  TUnitNode   = CLASS(TTreeNode)
  PRIVATE
    FsName    : STRING;
    FbVisible : BOOLEAN;
    FbProcessd: BOOLEAN;  {1991.0224}
  PUBLIC
    CONSTRUCTOR Create(asUnitName:STRING; abInInterface:BOOLEAN);
  END {CLASS};

  DNtreeAction= PROCEDURE(aoSon,aoFather:TTreeNode);

  TUnitTree   = CLASS(TTree)
  PRIVATE
    Level     : WORD;
    Leftable  : BOOLEAN;
    Size      : WORD;
  PUBLIC
    CONSTRUCTOR Create;
    FUNCTION    Compare(avKey1,avKey2:VARIANT):INTEGER; OVERRIDE;
    FUNCTION    DepthFirstFind(avKey:VARIANT):TTreeNode;
    FUNCTION    GetKey(aoNode:TTreeNode):VARIANT; OVERRIDE;
    FUNCTION    InsertLR(aoRght,aoNode:TTreeNode; achDir:CHAR):TTreeNode;
    PROCEDURE   Unravel(action:DNtreeAction);
    PROCEDURE   Unwind(action:TreeAction);
  END {CLASS};

CONST
  ciCenteringWidth = 100; 
VAR
  goUnits     : TUnitTree;   { Tree of unit dependencies}
  goFileList  : TStringList;

{}PROCEDURE   OpenTextFile(VAR aTxt:TEXTFILE; asFileName:STRING);
{}FUNCTION    SortByUnitName(aoList:TStringList; aiIndex1,aiIndex2:INTEGER):INTEGER;
{}PROCEDURE   Units_BuildTree(asFileName:STRING);
{}PROCEDURE   Units_ClearTree;
{}PROCEDURE   Write_ReportHdr(VAR aTEXT:TEXTFILE);
{}PROCEDURE   Write_UnitList(asReportFileName:STRING);
{}PROCEDURE   Write_UnitTree(asReportFileName:STRING);
              {=================================================}
IMPLEMENTATION
USES
  Contnrs,  SysUtils,
  KFrmtGlbl,  {lists of scanner tokens}
  KSSscaner,  {pascal source code scanner}
  KSSsymbols, {nodes to store scanner tokens}
  KSString,   {additional string routines}
  UnitDpnd,
  LstGlobal;

{.$DEFINE Debug}


{}CONSTRUCTOR TUnitNode.Create(asUnitName:STRING; abInInterface:BOOLEAN);
  BEGIN
    FsName    := asUnitName;
    FbVisible := abInInterface;
    FbProcessd:= FALSE;   {1991.0224}
{}END {TUnitNode.Create};


              {-------------------------------------------------}


{}CONSTRUCTOR TUnitTree.Create;
  BEGIN
    INHERITED Create;
    size := 0;       {1991.0224}
{}END {Create};




{}FUNCTION    TUnitTree.InsertLR(aoRght,aoNode:TTreeNode; achDir:CHAR):TTreeNode;
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
{}{}PROCEDURE Traverse(VAR aoRght:TTreeNode; aoNode:TTreeNode; acDir:CHAR);
    BEGIN
      IF aoRght=NIL THEN BEGIN              {empty subtree}
        aoRght := aoNode;                   {store node}
        aoRght.Left  := NIL;                {initialize sibling}
        aoRght.Right := NIL;                {initialize daughter}
        InsertLR := aoRght;                 {return insertion address}
      END ELSE
      IF acDir = 'L' THEN BEGIN             {sibling branch insertion}
        Traverse(aoRght.Left,aoNode,'L')
      END ELSE BEGIN                        {daughter branch insertion}
        Traverse(aoRght.Right,aoNode,'L')   { 1 right then successive lefts}
      END {IF};
{}{}END {Traverse};


  BEGIN
    achDir := UpCase(achDir);
    IF aoRght=NIL THEN BEGIN
      Traverse(Root,aoNode,achDir)
    END ELSE BEGIN
      Traverse(aoRght,aoNode,achDir);
    END {IF};

    Inc(Size);
{}END {TUnitTree.InsertLR};




{}PROCEDURE   TUnitTree.Unwind(action:TreeAction);
{+M
---------------------------------------------------------------------------
  Revised     - 1991.0212 (KSB) Initialized level to zero.
---------------------------------------------------------------------------}
{}{}PROCEDURE Traverse(aoNode:TTreeNode);
    VAR
      loNode  : TTreeNode;
    BEGIN
      IF aoNode <> NIL THEN BEGIN
        loNode := aoNode.Left;
        Leftable := loNode <> NIL;
        IF Assigned(action) THEN BEGIN
          Action(aoNode);
        END {IF};

        Inc(level);
        Traverse(aoNode.Right);
        Dec(level);
        Traverse(loNode);
      END {IF};
{}{}END {Traverse};


  BEGIN
    Level    := 0;
    Leftable := FALSE;
    Traverse(Root);
{}END {Unwind};




{}PROCEDURE   TUnitTree.Unravel(action:DNtreeAction);
{}{}PROCEDURE Traverse(aoSon,aoFather:TTreeNode);
    BEGIN
      IF aoSon <> NIL THEN BEGIN
        Action(aoSon,aoFather);
        Traverse(aoSon.Right,aoSon);
        Traverse(aoSon.Left,aoFather);
      END {IF};
{}{}END {Traverse};


  BEGIN
    Traverse(Root,NIL);
{}END {Unravel};




{}FUNCTION    TUnitTree.Compare(avKey1,avKey2:VARIANT):INTEGER;
  VAR
    ls1       : STRING;
    ls2       : STRING;
  BEGIN
    ls1 := UpperCase(avKey1);
    ls2 := UpperCase(avKey2);
    IF ls1 < ls2 THEN BEGIN
      result := -1
    END ELSE
    IF ls1 > ls2 THEN BEGIN
      result :=  1
    END ELSE BEGIN
      result :=  0;
    END {IF};
{}END {Compare};




{}FUNCTION    TUnitTree.GetKey(aoNode:TTreeNode):VARIANT;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := TUnitNode(aoNode).FsName;
{}END {TUnitTree.GetKey};




{}FUNCTION    TUnitTree.DepthFirstFind(avKey:VARIANT):TTreeNode;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
{}{}PROCEDURE Traverse(aoNode:TTreeNode);
    BEGIN
      IF aoNode <> NIL THEN BEGIN
        IF Compare(avKey,GetKey(aoNode)) = 0 THEN BEGIN
          result := aoNode
        END ELSE BEGIN
          IF aoNode.left <> NIL THEN BEGIN
            Traverse(aoNode.left);
          END {IF};

          IF aoNode.right <> NIL THEN BEGIN
            Traverse(aoNode.right);
          END {IF};
        END {IF};
      END {IF};
{}{}END {Traverse};


  BEGIN
    DepthFirstFind := NIL;
    Traverse(root);
{}END {DepthFirstFind};



              {=================================================}

CONST
  caHV        : ARRAY[BOOLEAN]OF CHAR = ('H','V');
  ch_Horiz    = '-';
  ch_Vert     = '|';
  ch_LTee     = '+';
  ch_LL       = '=';
  ciIndentation    = 5;
VAR
  usMainFileName   : STRING;           { File name of main program }
VAR
  uTxt        : TEXTFILE;
  usLine      : STRING;
  usLnPrev    : STRING;
  usLBranch   : STRING[5];
  usLEnd      : STRING[5];



{}FUNCTION    Structure(s:STRING; abLeft:BOOLEAN; aiDepth:WORD):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    lsLine    : STRING;
  BEGIN
    IF aiDepth > 0 THEN BEGIN
      lsLine := Copy(s,1,Pred(aiDepth)*ciIndentation)
    END ELSE BEGIN
      result := '';
      Exit;
    END {IF};

    CASE abLeft OF
      TRUE : lsLine := lsLine + usLBranch;
      ELSE   lsLine := lsLine + usLEnd;
    END {CASE};

    lsLine := ReplaceAll(lsLine,Trim(usLBranch)+'   ',ch_Vert+'    ');
    lsLine := ReplaceAll(lsLine,Trim(usLEnd)   +'   ','     ');
    result := lsLine;
{}END {Structure};




{}PROCEDURE   DoDListDiag(aoNode:TTreeNode);
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF Assigned(aoNode) THEN BEGIN
      WITH TUnitNode(aoNode) DO BEGIN
        usLine := Structure(usLnPrev,goUnits.Leftable,goUnits.Level);
        Write  (uTxt,usLine);
{$IFDEF Debug}
        Write  (uTxt,goUnits.Level:3);
{$ENDIF}
        Write  (uTxt,' ',caHV[FbVisible],':',FsName);
{$IFDEF Debug}
        IF Left <> NIL THEN BEGIN
          Write(uTxt,^I'  L:',goUnits.GetKey(Left));
        END ELSE BEGIN
          Write(uTxt,^I'  L:NIL');
        END {IF};

        IF Right <> NIL THEN BEGIN
          Write(uTxt,^I'  R:',goUnits.GetKey(Right));
        END ELSE BEGIN
          Write(uTxt,^I'  R:NIL');
        END {IF};
{$ENDIF}
        WriteLn(uTxt);
        usLnPrev := usLine;
      END {WITH};
    END {IF};
{}END {DoDListDiag};




TYPE
  TScanner    = CLASS(TKSSPasScanner)
  PRIVATE
    FbLnkJump : BOOLEAN;
    FoLink    : TBaseNode;
    FoNodes   : TObjectList;
  PUBLIC
    PROPERTY    Linked: BOOLEAN     READ FbLnkJump WRITE FbLnkJump;
    PROPERTY    Link  : TBaseNode   READ FoLink    WRITE FoLink;
    PROPERTY    Nodes : TObjectList READ FoNodes;
  PUBLIC
    CONSTRUCTOR Create;
    DESTRUCTOR  Destroy; OVERRIDE;
    FUNCTION    ScanCode:BOOLEAN; OVERRIDE;
  END {CLASS};




{}CONSTRUCTOR TScanner.Create;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create;
    FoNodes    := TObjectList.Create;
    FoLink     := NIL;
    FbLnkJump  := TRUE;
{}END {Create};




{}DESTRUCTOR  TScanner.Destroy;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FoNodes.Free;
{}END {Destroy};




{}FUNCTION    TScanner.ScanCode:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    le        : TTokenKinds;
    loIdent   : TIdentNode;
    loLitFlt  : TLitFloat;
    loLitInt  : TLitInteger;
    loLitStr  : TLitString;
    loSym     : TSymbolNode;
    loWord    : TPasWordNode;
    ls        : STRING;
  BEGIN
    result := FALSE;
    FoNodes.Clear;
    WHILE NOT IsEmpty DO BEGIN
      le := WhatKind(ls);

      CASE le OF
        tkFormattedBlock : BEGIN
        END {tkFormattedBlock};

        tkCommentSlash,
        tkCommentBrace,
        tkCommentPStar : BEGIN
        END {tkCommentBrace,tkCommentPStar};

        symAmpersand..symSlash : BEGIN
          loSym := TSymbolNode.Create;
          loSym.Symbol := le;
          FoNodes.Add(loSym);
        END {symAmpersand..symSlash};

        rwAnd..swOn : BEGIN
          loWord := TPasWordNode.Create;
          loWord.PWord := le;
          FoNodes.Add(loWord);
        END {rwAnd..swOn};

        tkIdentifier : BEGIN
          loIdent := TIdentNode.Create;
          loIdent.Value := ls;
          FoNodes.Add(loIdent);
        END {tkIdentifier};

        tkLiteralString : BEGIN
          loLitStr := TLitString.Create;
          loLitStr.Value := ls;
          FoNodes.Add(loLitStr);
        END {tkLiteralString};

        tkLiteralInteger : BEGIN
          loLitInt := TLitInteger.Create;
          loLitInt.Value := ls;
          FoNodes.Add(loLitInt);
        END {tkLiteralInteger};

        tkLiteralFloat : BEGIN
          loLitFlt := TLitFloat.Create;
          loLitFlt.Value := ls;
          FoNodes.Add(loLitFlt);
        END {tkLiteralFloat};
      END {CASE};
    END {WHILE};

    result := TRUE;
{}END {ScanCode};




{}FUNCTION    FoundInSubdirectory(asPath,asFileName:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
    lrSearch  : TSearchRec;
    lsPath    : STRING;
  BEGIN
    result := asFileName;
    IF FileExists(asPath+asFileName) THEN BEGIN
      result := ExpandFileName(asPath + asFileName);
    END ELSE
    IF FindFirst(AddBackSlash(asPath,'*'),faDirectory,lrSearch)=0 THEN BEGIN
      lsPath := AddBackSlash(asPath,lrSearch.Name);
      IF FileExists(AddBackSlash(lsPath,asFileName)) THEN BEGIN
        result := ExpandFileName(AddBackSlash(asPath,asFileName));
        FindClose(lrSearch);
        Exit;
      END ELSE BEGIN
        REPEAT
          li := FindNext(lrSearch);
          IF (li = 0) AND ((lrSearch.Attr AND faDirectory) = faDirectory) THEN BEGIN
            lsPath := AddBackSlash(asPath,lrSearch.Name);
            IF FileExists(AddBackSlash(lsPath,asFileName)) THEN BEGIN
              result := ExpandFileName(AddBackSlash(lsPath,asFileName));
              Break;
            END {IF};
          END {IF};
        UNTIL li <> 0;
      END {IF};

      FindClose(lrSearch);
    END {IF};
{}END {FoundInSubdirectory};


VAR
  usThisPath: STRING;

{}FUNCTION    ExpandPath(asFileName:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := asFileName;
    IF ExtractFilePath(asFileName) = '' THEN BEGIN
      IF FileExists(asFileName) THEN BEGIN
        result := ExpandFileName(usThisPath+asFileName);
        Exit;
      END ELSE
      IF gbUseSearchP THEN BEGIN
        result := FileSearch(asFileName,gsSearchPath);
        IF result <> '' THEN BEGIN
          Exit;
        END ELSE BEGIN
          result := asFileName;
        END {IF};
      END {IF};

      IF (gsDelphi_VCL <> '') AND gbUseVCL THEN BEGIN
        result := FoundInSubdirectory(gsDelphi_VCL,asFileName);
        IF result <> asFileName THEN BEGIN
          Exit;
        END {IF};
      END {IF};

      IF (gsDelphi_RTL <> '') AND gbUseRTL THEN BEGIN
        result := FoundInSubdirectory(gsDelphi_RTL,asFileName);
        IF result <> asFileName THEN BEGIN
          Exit;
        END {IF};
      END {IF};
    END ELSE BEGIN
      result := ExpandFileName(asFileName);
    END {IF};
{}END {ExpandPath};




{}PROCEDURE   GetUsesList(aoFileList:TStringList);
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  LABEL         LRestart;
  VAR
    lbIsaUnit : BOOLEAN;
    lbVisible : BOOLEAN;
    lchBranch : CHAR;
    liNodes   : INTEGER;
    liFilePtr : INTEGER;
    loBNode   : TBaseNode;
    loPasScanr: TScanner;
    loSource  : TStringList;
    loUNode   : TUnitNode;
    loTRoot   : TTreeNode;
    lsFileName: STRING;
    lsUName   : STRING;
  BEGIN
    loUNode    := TUnitNode.Create(usMainFileName,TRUE);
    usThisPath := ExtractFilePath(usMainFileName);
    goUnits.InsertLR(goUnits.Root,loUNode,'L');
    loSource := TStringList.Create;
    loPasScanr := TScanner.Create;
    lbIsaUnit  := TRUE;
    lbVisible  := TRUE;
    lsFileName := usMainFileName;
    liFilePtr  := 0;
    lchBranch  := 'L';
    aoFileList.Objects[liFilePtr] := POINTER(TRUE);
    TRY
LRestart:
      IF FileExists(lsFileName) THEN BEGIN
        loPasScanr.SrcBuffer.Clear;
        loSource.LoadFromFile(lsFileName);
        loPasScanr.Source := loSource;
        IF loPasScanr.ScanCode THEN BEGIN
          liNodes := 0;
          WHILE liNodes < loPasScanr.Nodes.Count-1 DO BEGIN
            loBNode := TBaseNode(loPasScanr.Nodes.Items[liNodes]);
            IF loBNode.IsWord THEN BEGIN
              IF (loBNode IS TPasWordNode) THEN BEGIN
                CASE TPasWordNode(loBNode).PWord OF
                  rwProgram        : lbIsaUnit := FALSE;
                  rwLibrary        : lbIsaUnit := FALSE;
                  rwImplementation : lbVisible := FALSE;
                  rwUses : BEGIN
                    lchBranch := 'R';
                    REPEAT
                      Inc(liNodes);
                      lsUName := TIdentNode(loPasScanr.Nodes.Items[liNodes]).Value;
                      loUNode := TUnitNode.Create(lsUName,lbVisible);
                      loTRoot := goUnits.DepthFirstFind(ChangeFileExt(ExtractFileName(lsFileName),''));
                      IF loTRoot = NIL THEN BEGIN
                        lchBranch := 'L';
                        loTRoot := goUnits.Root;
                      END {IF};

                      lsUName := lsUName + '.pas';
                      goUnits.InsertLR(loTRoot,loUNode,lchBranch);
                      REPEAT
                        Inc(liNodes);
                        IF (TBaseNode(loPasScanr.Nodes.Items[liNodes]) IS TPasWordNode)
                        AND(TPasWordNode(loPasScanr.Nodes.Items[liNodes]).PWord = rwIn) THEN BEGIN
                          lsUName := TValueNode(loPasScanr.Nodes.Items[liNodes+1]).Value;
                          Delete(lsUName,1,1);
                          Delete(lsUName,Length(lsUName),1);
                        END {IF};
                      UNTIL (TBaseNode(loPasScanr.Nodes.Items[liNodes]) IS TSymbolNode)
                      AND (TSymbolNode(loPasScanr.Nodes.Items[liNodes]).Symbol IN [symSemiColon,symComma]);

              {add file name to list of all files}
                      lsUName := ExpandPath(lsUName);
                      IF aoFileList.IndexOf(lsUName) = -1 THEN BEGIN
                        aoFileList.Add(lsUName);
                      END {IF};
                    UNTIL (TSymbolNode(loPasScanr.Nodes.Items[liNodes]).Symbol = symSemiColon);
                  END {rwUses};
                END {CASE};
              END
            END {IF};

            Inc(liNodes);
          END {WHILE};
        END {IF};
      END {IF};

      loPasScanr.Nodes.Clear;
      loSource.Clear;
      aoFileList.Sort;
      liFilePtr := 0;
      lchBranch := 'L';
      WHILE liFilePtr < aoFileList.Count DO BEGIN
        IF NOT LongBOOL(aoFileList.Objects[liFilePtr]) THEN BEGIN
          lsFileName := aoFileList[liFilePtr];
          aoFileList.Objects[liFilePtr] := POINTER(TRUE);
          lbIsaUnit := TRUE;
          lbVisible := TRUE;
          GOTO LRestart;
        END ELSE BEGIN
          Inc(liFilePtr);
        END {IF};
      END {WHILE};
    FINALLY
      FreeAndNil(loPasScanr);
      FreeAndNil(loSource);
    END {TRY};
{}END {GetUsesList};



VAR
  usBanner    : STRING;


{}PROCEDURE   Write_ReportHdr(VAR aTEXT:TEXTFILE);
  BEGIN
    WriteLn(aTEXT,CenterStr(usBanner,ciCenteringWidth));
    WriteLn(aTEXT);
{}END {Write_ReportHdr};



{}PROCEDURE   OpenTextFile(VAR aTxt:TEXTFILE; asFileName:STRING);
  BEGIN
    Assign(aTxt,asFileName);
    IF FileExists(asFileName) THEN BEGIN
      Append(aTxt);
    END ELSE BEGIN
      Rewrite(aTxt);
    END {IF};
{}END {OpenTextFile};




{}FUNCTION    SortByUnitName(aoList:TStringList; aiIndex1,aiIndex2:INTEGER):INTEGER;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    ls1       : STRING;
    ls2       : STRING;
  BEGIN
    ls1 := UpperCase(ExtractFileName(aoList[aiIndex1]));
    ls2 := UpperCase(ExtractFileName(aoList[aiIndex2]));
    IF ls1 < ls2 THEN BEGIN
      result := -1;
    END ELSE
    IF ls1 > ls2 THEN BEGIN
      result := +1;
    END ELSE BEGIN
      result := 0;
    END {IF};
{}END {SortByUnitName};




{}PROCEDURE   Write_UnitList(asReportFileName:STRING);
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    lbCloseIt : BOOLEAN;
    li        : INTEGER;
    lrSearch  : TSearchRec;
    lsSize    : STRING[13];
    lsDate    : STRING[19];
    lTxt      : TEXTFILE;
  BEGIN
    OpenTextFile(lTxt,asReportFileName);
    usBanner := 'W32 DELPHI TOOL REPORT '+FormatDateTime('yyyy mmm dd @ hh:mm',Now);
    Write_ReportHdr(lTxt);
    WriteLn(lTxt);
    WriteLn(lTxt,CenterStr('Unit File Listing for '+usMainFileName,ciCenteringWidth));
    WriteLn(lTxt);
    WriteLn(lTxt,'Source Files:');
    WriteLn(lTxt,'  No.  Unit Name            File Size      Last Changed         File Name');
    WriteLn(lTxt,'-----  -------------------- -------------  -------------------  ------------');
    lbCloseIt := FALSE;

    goFileList.CustomSort(SortByUnitName);
    FOR li := 0 TO goFileList.Count-1 DO BEGIN
      IF FileExists(goFileList[li]) THEN BEGIN
        IF FindFirst(goFileList[li],faAnyFile,lrSearch) = 0 THEN BEGIN
          lbCloseIt := TRUE;
          lsSize := LeftPadCh(FormatFloat('#,###,###,###',lrSearch.Size),' ',13);
          lsDate := FormatDateTime('yyyy.mm.dd hh:mm:ss',FileDateToDateTime(lrSearch.Time));
        END ELSE BEGIN
          lsSize := StringOfChar('.',13);
          lsDate := StringOfChar('.',19);
        END {IF};
      END ELSE BEGIN
        lsSize := StringOfChar('.',13);
        lsDate := StringOfChar('.',19);
      END {IF};

      WriteLn(lTxt,li+1:5,': ',Pad(ChangeFileExt(ExtractFileName(goFileList[li]),''),20),' ',lsSize,'  ',lsDate,'  ',goFileList[li]);
    END {FOR};

    goFileList.Sort;
    IF lbCloseIt THEN BEGIN
      FindClose(lrSearch);
    END {IF};

    WriteLn(lTxt);
    WriteLn(lTxt);
    Flush(lTxt);
    CloseFile(lTxt);
{}END {Write_UnitList};




{}PROCEDURE   Write_UnitTree(asReportFileName:STRING);
{+H
---------------------------------------------------------------------------
  Purpose     - Draw a unit dependency tree.
---------------------------------------------------------------------------}
  BEGIN
    OpenTextFile(uTxt,asReportFileName);
    Write_ReportHdr(uTxt);
    WriteLn(uTxt);
    WriteLn(uTxt,CenterStr('Unit Tree Chart for '+usMainFileName,ciCenteringWidth));
    WriteLn(uTxt);

    usLine    := '';
    usLnPrev  := '';
    usLBranch := '   '+ch_LTee + ch_Horiz;
    usLEnd    := '   '+ch_LL   + ch_Horiz;

    goUnits.Unwind(DoDListDiag);
    Flush(uTxt);
    CloseFile(uTxt);
{}END {Write_UnitTree};





{}PROCEDURE   Units_BuildTree(asFileName:STRING);
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    Units_ClearTree;
    goFileList.Add(asFileName);
    usMainFileName := asFileName;
    GetUsesList(goFileList);
{}END {Units_BuildTree};




{}PROCEDURE   Units_ClearTree;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    goFileList.Clear;
    goUnits.Clear;
{}END {Units_ClearTree};




INITIALIZATION
  goUnits := TUnitTree.Create;
  goFileList := TStringList.Create;
  usBanner := 'W32 DELPHI TOOL REPORT';
FINALIZATION
  FreeAndNil(goUnits);
  FreeAndNil(goFileList);
END {UnitTree}.
